/***********************************************************************/
/*!
 *  \file       player2.c
 *  \brief      demo player for playing PT3 using SDCC
 *  \author     Shining \n
 *           
 * 	\b Details:  This is the version NOT USING interrupt for playback and
 *				 reading keyboard.
 *				 For version with irq and equalizer see player.c.
 *
 *  \b Project: Playcity PT3-Support for SDCC \n
 *
 *  \date       20.12.2014 - File created
 *  \date       00.00.2010 - yy
 *  
 *  \version 1.0
 *
 ***********************************************************************/
 
#include <string.h>
#include "playcityPT3.h"	// The play-lib
#include "GphStr.h"			// Part of CPCRSLIB for printing 
#include "player.h"


////////////////////////////////////////////////////////////////////////
// Local variables
////////////////////////////////////////////////////////////////////////
unsigned char aKeyboard[KEYBOARD_HW_LINES];		//Array to handle the 80 physical keys bit state
unsigned char aKeyboardEntprell[Key_Max];		// Debounce - variable for keys

// These two variables save the content of cpc-system-irq when disabling it
unsigned char ucJumptableSave1;			
unsigned short ucJumptableSave2;

// Screen-Addresses for print
const unsigned int blocks[] = 
		{0xC000, 0xC050, 0xC0A0, 0xC0F0, 0xC140, 0xC190, 0xC1E0, 0xC230, 0xC280, 0xC2D0, 0xC320, 0xC370, 0xC3C0, 0xC410, 0xC460, 0xC4B0, 0xC500, 0xC550, 0xC5A0, 0xC5F0, 0xC640, 0xC690, 0xC6E0, 0xC730, 0xC780};
	
	
	
/* ++++++++++++++++++++++++++++++++++++++++++++++++++ */
/*! 
 * \brief       Set inks for print-routine
 *
 * \b Details:  Set ink for print-routine
 *
 * \param       a which ink
 * \param       b which colour
 *
 * \retval      --
 */
/* ++++++++++++++++++++++++++++++++++++++++++++++++++ */
void cpc_SetInkGphStr(char a, char b)
{
	a;
	b;
	__asm
	
	LD A,5 (IX) ;VALOR
	LD C,4 (IX)	;COLOR

	LD HL,#colores_b0
	LD B,#0
	ADD HL,BC
	LD (HL),A
	
	__endasm;
}


/* ++++++++++++++++++++++++++++++++++++++++++++++++++ */
/*! 
 * \brief       Read CPC keyboard
 *
 * \b Details:  Reads the cpc-keyboard into variable 
 *				aKeyboard using hardware.
 *
 * \param       --
 *
 * \retval      --
 */
/* ++++++++++++++++++++++++++++++++++++++++++++++++++ */
void ReadKeyboard(void)
{
  __asm
  
    ld hl,#_aKeyboard        ; buffer to store matrix data

	ld bc,#0xf40e                ; write PSG register index (14) to PPI port A (databus to PSG)
	out (c),c

	ld b,#0xf6
	in a,(c)
	and #0x30
	ld c,a

	or #0xC0                     ; bit 7=bit 6=1 (PSG operation: write register index)
	out (c),a                  ; set PSG operation -> select PSG register 14

	;; at this point PSG will have register 14 selected.
	;; any read/write operation to the PSG will act on this register.

	out (c),c                  ; bit 7=bit 6=0 (PSG operation: inactive)

	inc b
	ld a,#0x92
	out (c),a                  ; write PPI control: port A: input, port B: input, port C upper: output
							   ; port C lower: output
	push bc
	set 6,c                    ; bit 7=0, bit 6=1 (PSG operation: read register data)

	scan_key :
	ld b,#0xf6 
	out (c),c                 ;set matrix line & set PSG operation

	ld b,#0xf4                  ;PPI port A (databus to/from PSG)
	in a,(c)                  ;get matrix line data from PSG register 14

	cpl                       ;invert data: 1->0, 0->1
							  ;if a key/joystick button is pressed bit will be "1"
							  ;keys that are not pressed will be "0"

	ld (hl),a                 ;write line data to buffer
	inc hl                    ;update position in buffer
	inc c                     ;update line

	ld a,c
	and #0x0f
	cp #0x0a                    ;scanned all rows?
	jr nz,scan_key            ;no loop and get next row

	;; scanned all rows
	pop bc

	ld a,#0x82                  ;write PPI Control: Port A: Output, Port B: Input, Port C upper: output, Port C lower: output.
	out (c),a

	dec b
	out (c),c                 ;set PSG operation: bit7=0, bit 6=0 (PSG operation: inactive)
	
  __endasm;
} // void ReadKeyboard(void)





/* ++++++++++++++++++++++++++++++++++++++++++++++++++ */
/*! 
 * \brief       Check if key is pressed
 *
 * \b Details:  When keyboard was read using ReadKeyboard
 *				you can check with this routine if a specific
 *				key was pressed.
 *
 * \param       key 	which key to test
 *
 * \retval      TRUE: 	Key was pressed 
 * \retval      FALSE: 	Key was NOT pressed 
 */
/* ++++++++++++++++++++++++++++++++++++++++++++++++++ */
unsigned char IsKeyPressed(enum E_Key key)
{
  unsigned char nKeyLine = key / 8;
  unsigned char nBit = key % 8;
  
  if(((unsigned char)1 << nBit & aKeyboard[nKeyLine]) == 0)
    return TRUE;
  
  return FALSE;
} // unsigned char IsKeyPressed(enum E_Key key)


/* ++++++++++++++++++++++++++++++++++++++++++++++++++ */
/*! 
 * \brief       Check if key is pressed with debounce
 *
 * \b Details:  Like IsKeyPressed but keys will be debounced
 *
 * \param       key 	which key to test
 *
 * \retval      TRUE: 	Key was pressed 
 * \retval      FALSE: 	Key was NOT pressed 
 */
/* ++++++++++++++++++++++++++++++++++++++++++++++++++ */
unsigned char IsKeyPressed2(enum E_Key key)
{
  unsigned char nKeyLine = key / 8;
  unsigned char nBit = key % 8;
  
  if(((unsigned char)1 << nBit & aKeyboard[nKeyLine]) == 0)
  {
	if(0==aKeyboardEntprell[key])
	{
		aKeyboardEntprell[key] = 2;
		return TRUE;
	}
  }
  else
  {
	if(aKeyboardEntprell[key]>0)
	{
		aKeyboardEntprell[key]--;
	}
  }
  
  return FALSE;
} // unsigned char IsKeyPressed2(enum E_Key key)


/* ++++++++++++++++++++++++++++++++++++++++++++++++++ */
/*! 
 * \brief       Searches for PT3-files in a catalogue
 *
 * \b Details:  In variable ucCATalogue, a CPC-standard-
 *				catalogue will be provided. Method searches for
 *				all files ending with .pt3 and provides them in
 *				a list (ucCATValidPT3). The routine reads only 100
 *				filenames.
 *
 * \param       --
 *
 * \retval      number of found pt3-files
 */
/* ++++++++++++++++++++++++++++++++++++++++++++++++++ */
unsigned char SearchValidPT3(void)
{
	unsigned char i;
	unsigned char catJump=0;
	unsigned char countValidFiles=0;
	
	for(i=0;i<VALID_PT3_FILES;i++)
	{
		ucCATValidPT3[i] = 0xFF; // Invalid
	}
	
	for(i=0;i<100;i++)
	{
		if( 0 == strncmp(&ucCATalogue[(i*14)+9], "PT3", 3) )
		{
			ucCATValidPT3[catJump++] = i;
			countValidFiles++;
			if(VALID_PT3_FILES == catJump)
			{
				break;
			}
		}
 
	}
	
	return countValidFiles;
} // unsigned char SearchValidPT3(void)


/* ++++++++++++++++++++++++++++++++++++++++++++++++++ */
/*! 
 * \brief       CPC CAT command
 *
 * \b Details:  CPC CAT command
 *
 * \param       --
 *
 * \retval      --
 */
/* ++++++++++++++++++++++++++++++++++++++++++++++++++ */
void GetCATalogue(void)
{
	__asm
		ld de, #_ucCATalogue
		call #0xbc9b  ;;CAS CATALOG
	__endasm;
} // void GetCATalogue(void)


/* ++++++++++++++++++++++++++++++++++++++++++++++++++ */
/*! 
 * \brief       Enable CPC-ROMS for file-load
 *
 * \b Details:  Enable ROMS for file-load
 *
 * \param       --
 *
 * \retval      --
 */
/* ++++++++++++++++++++++++++++++++++++++++++++++++++ */
void SetupDOS()
{
  //Based on http://cpctech.cpc-live.com/source/loader.html
  __asm
	ld	hl,#0
	add	hl,sp
	ld	a,(hl)
	ld	e,a
	ld	hl,#1
	add	hl,sp
	ld	a,(hl)
	ld	d,a
    ld (_stack+1), de

    ;;------------------------------------------------------------------------
    ;; store the drive number the loader was run from
    ld hl,(#0xbe7d)
    ld a,(hl)                  
    ld (_drive+1),a

    ;;------------------------------------------------------------------------
    ld c,#0xff          ;; disable all roms
    ld hl, #_start222         ;; execution address for program
    call #0xbd16  ;;mc_start_program    ;; start it

    _start222:: nop

    call #0xbccb  ;;kl_rom_walk     ;; enable all roms 

    ;;------------------------------------------------------------------------
    ;; when AMSDOS is enabled, the drive reverts back to drive 0!
    ;; This will restore the drive number to the drive the loader was run from
    _drive: ld a, #0x00
    ld hl,(#0xbe7d)
    ld (hl),a     

    _stack: ld hl, #0x0000
    push hl //one for sdcc ix pop
    push hl //two for return address
  __endasm;
} // void SetupDOS()



/* ++++++++++++++++++++++++++++++++++++++++++++++++++ */
/*! 
 * \brief       Load a file
 *
 * \b Details:  Loads a file to a specific address
 *
 * \param       sFileName		name of the file to load
 * \param       pLoadAddress	address to load the file to
 * \param       nFileNameLen	length of the filename in chars
 *								(for saving some code)
 *
 * \retval      --
 */
/* ++++++++++++++++++++++++++++++++++++++++++++++++++ */
void LoadFile(char *sFileName, char *pLoadAddress, unsigned char nFileNameLen)
{
	sFileName;
	pLoadAddress;
	nFileNameLen;
	
	
  __asm
    ;; B = length of the filename in characters
    ld b, 8 (IX) ;nFileNameLen

    ;; HL = address of the start of the filename
    LD L, 4 (IX) ;sFileName
    LD H, 5 (IX) ;sFileName

    ;; DE = address of a 2k buffer
    ;; in disc mode: this buffer is not used when CAS IN DIRECT
    ;; firmware function is used, so it is safe to put it anywhere
    ;; you want.
    ld de, #0x0

    ;; firmware function to open a file for reading
    call #0xbc77 ;;cas_in_open
	ld (_songlen) , BC ;; Global var for song-len 

    ;; firmware function to load the entire file
    ;; this will work with files that have a AMSDOS header (ASCII
    ;; files do not have a header)
    ;; HL = load address
    LD L, 6 (IX) ;pLoadAddress
    LD H, 7 (IX) ;pLoadAddress

    ;; read file
    call #0xbc83 ;;cas_in_direct

    ;; firmware function to close a file opened for reading
    call #0xbc7a ;;cas_in_close

  __endasm;

} // void LoadFile(char *sFileName, char *pLoadAddress, unsigned char nFileNameLen)



/* ++++++++++++++++++++++++++++++++++++++++++++++++++ */
/*! 
 * \brief       Set screen-inks
 *
 * \b Details:  This routine sets inks on the screen NOT using
 *				firmware.
 *
 * \param       nColorIndex		Which ink
 * \param       nPalette		Which colour
 *
 * \retval      --
 */
/* ++++++++++++++++++++++++++++++++++++++++++++++++++ */
void SetColorDirect(unsigned char nColorIndex, unsigned char nPalette)
{
	nColorIndex;
	nPalette;

  __asm
    ld e, 4( ix )
	ld l, 5( ix )

	ld b, #0x7f
	ld c, e
	di
	out (c), c
	ld c, l
	out (c), c
	ei
  __endasm;
} // void SetColorDirect(unsigned char nColorIndex, unsigned char nPalette)


/* ++++++++++++++++++++++++++++++++++++++++++++++++++ */
/*! 
 * \brief       Set screen-inks
 *
 * \b Details:  This routine sets inks on the screen using
 *				firmware.
 *
 * \param       nColorIndex		Which ink
 * \param       nPaletteIndex	Which colour
 *
 * \retval      --
 */
/* ++++++++++++++++++++++++++++++++++++++++++++++++++ */
void SetColor(unsigned char nColorIndex, unsigned char nPaletteIndex)
{
	nColorIndex;
	nPaletteIndex;
	
  __asm
    ld a, 4 (ix)
    ld b, 5 (ix)
    ld c, b
    call #0xBC32 ;SCR SET INK
  __endasm;
} // void SetColor(unsigned char nColorIndex, unsigned char nPaletteIndex)



/* ++++++++++++++++++++++++++++++++++++++++++++++++++ */
/*! 
 * \brief       Set mode
 *
 * \b Details:  Sets cpc-mode using firmware
 *
 * \param       nMode		Which mode
 *
 * \retval      --
 */
/* ++++++++++++++++++++++++++++++++++++++++++++++++++ */
void SetMode(unsigned char nMode)
{
	nMode;
	
  __asm
    ld a, 4 (ix)
    call #0xBC0E ;SCR_SET_MODE
  __endasm;
} // void SetMode(unsigned char nMode)


/* ++++++++++++++++++++++++++++++++++++++++++++++++++ */
/*! 
 * \brief       Initialize mode 0
 *
 * \b Details:  Initialize mode 0
 *
 * \param       --
 *
 * \retval      --
 */
/* ++++++++++++++++++++++++++++++++++++++++++++++++++ */
void InitMode0()
{
  SetMode(0);

  //SCR SET BORDER 0
  __asm
    ld b, #0 ;black
    ld c, b
    call #0xBC38
  __endasm;
} // void InitMode0()


/* ++++++++++++++++++++++++++++++++++++++++++++++++++ */
/*! 
 * \brief       Wait for frame-flyback
 *
 * \b Details:  Wait for frame-flyback
 *
 * \param       --
 *
 * \retval      --
 */
/* ++++++++++++++++++++++++++++++++++++++++++++++++++ */
void WaitVBL()
{
__asm
WaitVBL:
	ld a, #0xF5
	in a, (#0xDB)
	rra
	jr nc, WaitVBL
__endasm;
	
} // void WaitVBL()


/* ++++++++++++++++++++++++++++++++++++++++++++++++++ */
/*! 
 * \brief       Searches for part-2 start-address
 *
 * \b Details:  Searches for part 2 start address of a 
 *				6-channel song. If not found returns
 *				part one start address as part 2 start address.
 *				Remark: This routine searches for string Vort or ProT at
 *				the beginning of a part. Best way to aceive this is to load a song in
 *				vortex-tracker and save it again.
 *				(Sometimes without doing so, part 2 has no header)
 *
 * \param       songPart1Pointer 	Where in RAM starts part one of the song
 * \param       songPart2Pointer 	Returns the start address of part 2
 *
 * \retval      TRUE:  Seems to be valid song
 * \retval      FALSE: No valid sing found
 */
/* ++++++++++++++++++++++++++++++++++++++++++++++++++ */
unsigned char SearchSongStartAddresses(unsigned char* songPart1Pointer, unsigned char** songPart2Pointer)
{
	unsigned short i;
	*songPart2Pointer = songPart1Pointer; // For 3-channel songs - play both chips the same
	
	if( (0 != strncmp(songPart1Pointer, "Vort", 4)) && (0 != strncmp(songPart1Pointer, "ProT", 4)))
	{
		return FALSE;
	}
	
	for(i=100;i<songlen;i++)
	{
		if( (0 == strncmp(songPart1Pointer+i, "Vort", 4)) || (0 == strncmp(songPart1Pointer+i, "ProT", 4)) )
		{
			*songPart2Pointer = &songPart1Pointer[i];
			return TRUE;
		}
	}
	
	return TRUE; // When we get here, it is a song for only 3 channels
} // unsigned char SearchSongStartAddresses(unsigned char* songPart1Pointer, unsigned char** songPart2Pointer)


/* ++++++++++++++++++++++++++++++++++++++++++++++++++ */
/*! 
 * \brief       Disable firmware interrupt
 *
 * \b Details:  Disables the firmware irq but saves the content
 *				for later restore.
 *
 * \param       --
 *
 * \retval      --
 */
/* ++++++++++++++++++++++++++++++++++++++++++++++++++ */
void DisableFirmware(void)
{
	__asm
		; Set a null interrupt handler
		DI
		LD 	 a,(#0x0038)
		ld (_ucJumptableSave1),a
		LD   HL,(#0x0039)
		ld (_ucJumptableSave2),HL
		LD   A,#0xFB              ; EI (1)
		LD   HL,#0x4DED           ; RETI (4)
		LD   (#0x0038),A
		LD   (#0x0039),HL
		EI		
	__endasm;
} // void DisableFirmware(void)


/* ++++++++++++++++++++++++++++++++++++++++++++++++++ */
/*! 
 * \brief       Reenable firmware interrupt
 *
 * \b Details:  Reenables the firmware irq with the 
 *				previous stored values.
 *
 * \param       --
 *
 * \retval      --
 */
/* ++++++++++++++++++++++++++++++++++++++++++++++++++ */
void ReenableFirmware(void)
{
	__asm
		DI
		ld a,(_ucJumptableSave1)
		LD   (#0x0038),A
		ld HL,(_ucJumptableSave2)
		LD   (#0x0039),HL
		EI		
	__endasm;
} // void ReenableFirmware(void)


/* ++++++++++++++++++++++++++++++++++++++++++++++++++ */
/*! 
 * \brief       Loads a PT3 File
 *
 * \b Details:  This routine loads a PT3-file using the
 *				previous saved catalogue and the valid-file-list
 *				previous stored values.
 *
 * \param       index			index in the falid file list to load
 *				pLoadAddress	where to load the file to
 *
 * \retval      --
 */
/* ++++++++++++++++++++++++++++++++++++++++++++++++++ */
void LoadPT3File(unsigned char index,char *pLoadAddress)
{
	unsigned char ucFilename[12];
	
	if(0xFF != ucCATValidPT3[index]) // Filename ends with PT3 ?
	{
		strncpy(ucFilename,&ucCATalogue[(ucCATValidPT3[index]*14)+1],8);
		ucFilename[8]  = '.';
		ucFilename[9]  = 'P';
		ucFilename[10] = 'T';
		ucFilename[11] = '3';
		
		ReenableFirmware();

		LoadFile(ucFilename,pLoadAddress,12);
		
		// Search Part 2 of the Playcity 6-Channel-Song
		if(0==SearchSongStartAddresses(pLoadAddress, &songPart2StartAddress))
		{
			while(1); // Only for demonstration. In case of error this hangs the system
		}
		
		// Disable Firmware again
		DisableFirmware();

		// Init the song
		PC_Init(songPart1StartAddress,songPart2StartAddress);
	}
	
} // void LoadPT3File(unsigned char index,char *pLoadAddress)


/* ++++++++++++++++++++++++++++++++++++++++++++++++++ */
/*! 
 * \brief       Print text on the screen
 *
 * \b Details:  Print a text on x-y-coordinates on the
 *				screen using part of cpcrslib.
 *
 * \param       text	text to print
 * \param       x		x-coordinate
 * \param       y		y-coordinate		
 *
 * \retval      --
 */
/* ++++++++++++++++++++++++++++++++++++++++++++++++++ */
void PrintGphStrXY(char *text, char x, char y)
{		
	unsigned int buffer = blocks[y] + x*2;

	cpc_PrintGphStr(text,buffer);
} // void PrintGphStrXY(char *text, char x, char y)

void my_uitoa(unsigned short zahl, char* string) 
{
  char i;                            
 
  string[2]='\0';                    // String-terminator
  for(i=1; i>=0; i--) 
  {
    string[i]=(zahl % 10) +'0';       // Modulo rechnen, dann den ASCII-Code von '0' addieren
    zahl /= 10;
  }
}

/* ++++++++++++++++++++++++++++++++++++++++++++++++++ */
/*! 
 * \brief       main
 */
/* ++++++++++++++++++++++++++++++++++++++++++++++++++ */
void main()
{
	unsigned char i = 0;				// Generic count-var
	unsigned char loadSong = 0xff; 		// Which song to load. >9 no song
	static unsigned char filename[9];	// Holds song-filenames
	unsigned char songloaded = 0; 		// If 1, we call player
	static unsigned char text[4];
	unsigned char songsFound= 0;
	
	songPart1StartAddress = (unsigned char*)0x4000; // We load all Songs to 0x4000

	// Give Keyboard-Var something to debounce keys
	for(i=0;i<Key_Max;i++)
	{
		aKeyboardEntprell[i] = 2;
	}
	
	// Enable CPC-DOS
	SetupDOS();
	
	// Mode 0
	InitMode0();
	
	// All colours black using firmware
	for(i = 0; i < NUM_COLORS; i++)
	{
		SetColor(i, 0);
	}
	
	// Equal to cat in basic ( including printing to screen )
	GetCATalogue();
	
	// Search the CATalogue for .pt3 files
	songsFound = SearchValidPT3();
	
	//songsFound = hex2bin(&songsFound);
	
	// Coz cat printed something, reinit mode 0 (CLS)
	InitMode0();
	
	// Set Text-Colours
	cpc_SetInkGphStr(0,0);
	cpc_SetInkGphStr(1,2);
	cpc_SetInkGphStr(2,8);
	cpc_SetInkGphStr(3,2);
	//cpc_SetInkGphStr(2,2);
	//cpc_SetInkGphStr(2,2);
	
	// Print something on screen
	PrintGphStrXY("----------------------------------------",0,0);
	PrintGphStrXY("----------------------------------------",0,22);
	PrintGphStrXY("LITTLE;PLAYCITY;PT3;PLAYER;FOR;SDCC",2,2);
	//text[0] = (songsFound&0xff)+0x30;
	//text[1] = (songsFound>>4)+0x30;
	//text[2] = 0;
	my_uitoa(songsFound,text);
	PrintGphStrXY("FOUND",12,5);
	PrintGphStrXY(text,18,5);
	PrintGphStrXY("SONGS",21,5);
	PrintGphStrXY("ACTUAL;FILE:",9,8);
	PrintGphStrXY("CODE;BY;SHINING;IN;2K15",8,12);
	PrintGphStrXY("BASED;UPON;PT3;TURBO;SOUND;PLAYER;BY",3,16);
	PrintGphStrXY("S<V<BULBA;SYX;AND;PRODATRON",8,17);
	PrintGphStrXY("PRESS;0@9;AND;F0@F9;TO;PLAY;THE;SONGS",3,20);

	// Disable IRQ
	DisableFirmware();
	

	// First of all we load song 0
	loadSong = 0;
	
	
	
	// Main loop
	// Endless playback
	while(1)
	{
		WaitVBL(); // Wait for frame-flyback
		
		if(songloaded) // Not 0, we call the player
		{
			PC_Play();
		}
		
		ReadKeyboard(); // Read the CPC-Keyboard
		
		
		if(IsKeyPressed2(Key_0))
		{
			loadSong = 0;
		}
		if(IsKeyPressed2(Key_1))
		{
			loadSong = 1;
		}
		if(IsKeyPressed2(Key_2))
		{
			loadSong = 2;
		}
		if(IsKeyPressed2(Key_3))
		{
			loadSong = 3;
		}
		if(IsKeyPressed2(Key_4))
		{
			loadSong = 4;
		}
  
		if(IsKeyPressed2(Key_5_Joy2Down))
		{
			loadSong = 5;
		}
		if(IsKeyPressed2(Key_6_Joy2Up))
		{
			loadSong = 6;
		}
		if(IsKeyPressed2(Key_7))
		{
			loadSong = 7;
		}
		if(IsKeyPressed2(Key_8))
		{
			loadSong = 8;
		}
		if(IsKeyPressed2(Key_9))
		{
			loadSong = 9;
		}		
		if(IsKeyPressed2(Key_F0))
		{
			loadSong = 10;
		}
		if(IsKeyPressed2(Key_F1))
		{
			loadSong = 11;
		}
		if(IsKeyPressed2(Key_F2))
		{
			loadSong = 12;
		}
		if(IsKeyPressed2(Key_F3))
		{
			loadSong = 13;
		}
		if(IsKeyPressed2(Key_F4))
		{
			loadSong = 14;
		}
		if(IsKeyPressed2(Key_F5))
		{
			loadSong = 15;
		}
		if(IsKeyPressed2(Key_F6))
		{
			loadSong = 16;
		}
		if(IsKeyPressed2(Key_F7))
		{
			loadSong = 17;
		}
		if(IsKeyPressed2(Key_F8))
		{
			loadSong = 18;
		}
		if(IsKeyPressed2(Key_F9))
		{
			loadSong = 19;
		}
		
		
		if( (loadSong<VALID_PT3_FILES) && (0xFF != ucCATValidPT3[loadSong])) // We have to load a song
		{
			PC_Stop(); // Stop previous song
			LoadPT3File(loadSong,songPart1StartAddress);
			strncpy(filename,&ucCATalogue[(ucCATValidPT3[loadSong]*14)+1],8);
			
			
			// Replace space with ; for font
			for(i=0;i<8;i++)
			{
				if(filename[i] == ' ')
				{
					filename[i] = ';';
				}
			}
			filename[8] = '\0';
			
			PrintGphStrXY(filename,23,8);
			
			// Now set all colours to the desired ones NOT using firmware (cause irq is off ;))
			for(i = 0; i < NUM_COLORS; i++)
			{
				SetColorDirect(i, FirmwarePalette[Palette[i]]);
			}
				
				loadSong = 0xFF; // Disable song-load 
			}
		
			songloaded = 1; // Now we play
	}
	
}